;+
;PROCEDURE:	thm_load_esa_pot
;PURPOSE:	
;	Add spacecraft potential data to the ESA structures
;INPUT:		
;
;KEYWORDS:
;	sc:		string		themis spacecraft - "a", "b", "c", "d", "e"
;					if not set defaults to "a"		
;	themishome:	string		path to data dir, where data dir contains the th* dir, where *=a,b,c,d,e
;	datatype	string or 0/1	if not set, uses spin averaged moment data
;					if set to 1, uses vaf v12 data
;					if string, uses string data type - mom or v12, v34, v1234 where the latter are vaf data
;					vaf data is averaged by "time_average" with resolution=3s before interpolation 
;					s/c potential data are interpolated to center of esa data collection
;	pot_scale:	real		scale factor applied to measured probe potential, default=1.15
;	offset:		real		probe to plasma potential offset, default=1.0
;	min_pot:	real		minimum potential allowed, default=2.0
;	make_plot:	0/1		if set, makes tplot structure of potential 
;
;CREATED BY:	J. McFadden
;VERSION:	1
;LAST MODIFICATION:  07/07/23
;MOD HISTORY:
;
;NOTES:	  
;	
;-

pro thm_load_esa_pot,sc=sc,themishome=themishome,datatype=datatype,pot_scale=pot_scale,offset=offset,min_pot=min_pot,make_plot=make_plot

; sc default
	if not keyword_set(sc) then begin
		print,'S/C number not set, default = all probes'
		sc=['a','b','c','d','e','f']
	endif

	if not keyword_set(themishome) then themishome=!themis.local_data_dir

nsc = n_elements(sc)
probes=strarr(1)
if nsc eq 1 then probes[0]=sc
if nsc ne 1 then probes=sc

if not keyword_set(pot_scale) then scale=1.15 else scale=pot_scale
if not keyword_set(offset) then offset=1.0
if not keyword_set(min_pot) then min_pot=2.0

;***********************************************************************************
; get s/c potential data
; moment potential must be time shifted
; 1.6028 = 1 + 217/360 times spin_period is the spin offset time between s/c pot in moments and actual time s/c potential calculated

for i=0,nsc-1 do begin

	if not keyword_set(datatype) then begin
		thm_load_mom,probe=probes[i]
		get_data,'th'+probes[i]+'_pxxm_pot',data=tmp
		tshft_mom=1.6028
	endif else if string(datatype) eq 'mom' then begin
		thm_load_mom,probe=probes[i]
		get_data,'th'+probes[i]+'_pxxm_pot',data=tmp
		tshft_mom=1.6028
	endif else begin
		thm_load_efi,probe=probes[i],datatype='vaf'
		get_data,'th'+probes[i]+'_vaf',data=tmp1
		vaf12 = -0.5*reform(tmp1.y[*,0]+tmp1.y[*,1])
		vaf34 = -0.5*reform(tmp1.y[*,2]+tmp1.y[*,3])
		vaf1234 = -.25*reform(tmp1.y[*,0]+tmp1.y[*,1]+tmp1.y[*,2]+tmp1.y[*,3])
		vaf1234_3s=time_average(tmp1.x,vaf1234,resolution=3.,newtime=newtime)
		if keyword_set(make_plot) then store_data,'th'+probes[i]+'_v1234_3s',data={x:newtime,y:vaf1234_3s}
		vaf3s = vaf1234_3s
		vaf12_3s=time_average(tmp1.x,vaf12,resolution=3.,newtime=newtime)
		if keyword_set(make_plot) then store_data,'th'+probes[i]+'_v12_3s',data={x:newtime,y:vaf12_3s}
		if string(datatype) eq 'v12' then vaf3s = vaf12_3s
		vaf34_3s=time_average(tmp1.x,vaf34,resolution=3.,newtime=newtime)
		if keyword_set(make_plot) then store_data,'th'+probes[i]+'_v34_3s',data={x:newtime,y:vaf34_3s}
		if string(datatype) eq 'v34' then vaf3s = vaf34_3s
		if keyword_set(make_plot) then store_data,'th'+probes[i]+'_vaf_3s',data={x:newtime,y:vaf3s}
		tmp={x:newtime,y:vaf3s}
		tshft_mom=0.
	endelse

	if probes[i] eq 'a' then begin
		common tha_454,tha_454_ind,tha_454_dat 
		if n_elements(tha_454_dat) ne 0 then begin
		  if tha_454_ind ne -1 then begin
			tt=(tha_454_dat.time+tha_454_dat.end_time)/2. & time=tt+tshft_mom*tha_454_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			tha_454_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'tha_peif_sc_pot',data={x:tt,y:tha_454_dat.sc_pot}
		  endif
		endif
		common tha_455,tha_455_ind,tha_455_dat 
		if n_elements(tha_455_dat) ne 0 then begin
		  if tha_455_ind ne -1 then begin
			tt=(tha_455_dat.time+tha_455_dat.end_time)/2. & time=tt+tshft_mom*tha_455_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			tha_455_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'tha_peir_sc_pot',data={x:tt,y:tha_455_dat.sc_pot}
		  endif
		endif
		common tha_456,tha_456_ind,tha_456_dat 
		if n_elements(tha_456_dat) ne 0 then begin
		  if tha_456_ind ne -1 then begin
			tt=(tha_456_dat.time+tha_456_dat.end_time)/2. & time=tt+tshft_mom*tha_456_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			tha_456_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'tha_peib_sc_pot',data={x:tt,y:tha_456_dat.sc_pot}
		  endif
		endif
		common tha_457,tha_457_ind,tha_457_dat 
		if n_elements(tha_457_dat) ne 0 then begin
		  if tha_457_ind ne -1 then begin
			tt=(tha_457_dat.time+tha_457_dat.end_time)/2. & time=tt+tshft_mom*tha_457_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			tha_457_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'tha_peef_sc_pot',data={x:tt,y:tha_457_dat.sc_pot}
		  endif
		endif
		common tha_458,tha_458_ind,tha_458_dat 
		if n_elements(tha_458_dat) ne 0 then begin
		  if tha_458_ind ne -1 then begin
			tt=(tha_458_dat.time+tha_458_dat.end_time)/2. & time=tt+tshft_mom*tha_458_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			tha_458_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'tha_peer_sc_pot',data={x:tt,y:tha_458_dat.sc_pot}
		  endif
		endif
		common tha_459,tha_459_ind,tha_459_dat 
		if n_elements(tha_459_dat) ne 0 then begin
		  if tha_459_ind ne -1 then begin
			tt=(tha_459_dat.time+tha_459_dat.end_time)/2. & time=tt+tshft_mom*tha_459_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			tha_459_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'tha_peeb_sc_pot',data={x:tt,y:tha_459_dat.sc_pot}
		  endif
		endif
	endif else if probes[i] eq 'b' then begin
		common thb_454,thb_454_ind,thb_454_dat 
		if n_elements(thb_454_dat) ne 0 then begin
		  if thb_454_ind ne -1 then begin
			tt=(thb_454_dat.time+thb_454_dat.end_time)/2. & time=tt+tshft_mom*thb_454_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thb_454_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thb_peif_sc_pot',data={x:tt,y:thb_454_dat.sc_pot}
		  endif
		endif
		common thb_455,thb_455_ind,thb_455_dat 
		if n_elements(thb_455_dat) ne 0 then begin
		  if thb_455_ind ne -1 then begin
			tt=(thb_455_dat.time+thb_455_dat.end_time)/2. & time=tt+tshft_mom*thb_455_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thb_455_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thb_peir_sc_pot',data={x:tt,y:thb_455_dat.sc_pot}
		  endif
		endif
		common thb_456,thb_456_ind,thb_456_dat 
		if n_elements(thb_456_dat) ne 0 then begin
		  if thb_456_ind ne -1 then begin
			tt=(thb_456_dat.time+thb_456_dat.end_time)/2. & time=tt+tshft_mom*thb_456_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thb_456_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thb_peib_sc_pot',data={x:tt,y:thb_456_dat.sc_pot}
		  endif
		endif
		common thb_457,thb_457_ind,thb_457_dat 
		if n_elements(thb_457_dat) ne 0 then begin
		  if thb_457_ind ne -1 then begin
			tt=(thb_457_dat.time+thb_457_dat.end_time)/2. & time=tt+tshft_mom*thb_457_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thb_457_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thb_peef_sc_pot',data={x:tt,y:thb_457_dat.sc_pot}
		  endif
		endif
		common thb_458,thb_458_ind,thb_458_dat 
		if n_elements(thb_458_dat) ne 0 then begin
		  if thb_458_ind ne -1 then begin
			tt=(thb_458_dat.time+thb_458_dat.end_time)/2. & time=tt+tshft_mom*thb_458_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thb_458_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thb_peer_sc_pot',data={x:tt,y:thb_458_dat.sc_pot}
		  endif
		endif
		common thb_459,thb_459_ind,thb_459_dat 
		if n_elements(thb_459_dat) ne 0 then begin
		  if thb_459_ind ne -1 then begin
			tt=(thb_459_dat.time+thb_459_dat.end_time)/2. & time=tt+tshft_mom*thb_459_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thb_459_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thb_peeb_sc_pot',data={x:tt,y:thb_459_dat.sc_pot}
		  endif
		endif
	endif else if probes[i] eq 'c' then begin
		common thc_454,thc_454_ind,thc_454_dat 
		if n_elements(thc_454_dat) ne 0 then begin
		  if thc_454_ind ne -1 then begin
			tt=(thc_454_dat.time+thc_454_dat.end_time)/2. & time=tt+tshft_mom*thc_454_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thc_454_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thc_peif_sc_pot',data={x:tt,y:thc_454_dat.sc_pot}
		  endif
		endif
		common thc_455,thc_455_ind,thc_455_dat 
		if n_elements(thc_455_dat) ne 0 then begin
		  if thc_455_ind ne -1 then begin
			tt=(thc_455_dat.time+thc_455_dat.end_time)/2. & time=tt+tshft_mom*thc_455_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thc_455_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thc_peir_sc_pot',data={x:tt,y:thc_455_dat.sc_pot}
		  endif
		endif
		common thc_456,thc_456_ind,thc_456_dat 
		if n_elements(thc_456_dat) ne 0 then begin
		  if thc_456_ind ne -1 then begin
			tt=(thc_456_dat.time+thc_456_dat.end_time)/2. & time=tt+tshft_mom*thc_456_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thc_456_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thc_peib_sc_pot',data={x:tt,y:thc_456_dat.sc_pot}
		  endif
		endif
		common thc_457,thc_457_ind,thc_457_dat 
		if n_elements(thc_457_dat) ne 0 then begin
		  if thc_457_ind ne -1 then begin
			tt=(thc_457_dat.time+thc_457_dat.end_time)/2. & time=tt+tshft_mom*thc_457_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thc_457_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thc_peef_sc_pot',data={x:tt,y:thc_457_dat.sc_pot}
		  endif
		endif
		common thc_458,thc_458_ind,thc_458_dat 
		if n_elements(thc_458_dat) ne 0 then begin
		  if thc_458_ind ne -1 then begin
			tt=(thc_458_dat.time+thc_458_dat.end_time)/2. & time=tt+tshft_mom*thc_458_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thc_458_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thc_peer_sc_pot',data={x:tt,y:thc_458_dat.sc_pot}
		  endif
		endif
		common thc_459,thc_459_ind,thc_459_dat 
		if n_elements(thc_459_dat) ne 0 then begin
		  if thc_459_ind ne -1 then begin
			tt=(thc_459_dat.time+thc_459_dat.end_time)/2. & time=tt+tshft_mom*thc_459_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thc_459_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thc_peeb_sc_pot',data={x:tt,y:thc_459_dat.sc_pot}
		  endif
		endif
	endif else if probes[i] eq 'd' then begin
		common thd_454,thd_454_ind,thd_454_dat 
		if n_elements(thd_454_dat) ne 0 then begin
		  if thd_454_ind ne -1 then begin
			tt=(thd_454_dat.time+thd_454_dat.end_time)/2. & time=tt+tshft_mom*thd_454_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thd_454_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thd_peif_sc_pot',data={x:tt,y:thd_454_dat.sc_pot}
		  endif
		endif
		common thd_455,thd_455_ind,thd_455_dat 
		if n_elements(thd_455_dat) ne 0 then begin
		  if thd_455_ind ne -1 then begin
			tt=(thd_455_dat.time+thd_455_dat.end_time)/2. & time=tt+tshft_mom*thd_455_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thd_455_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thd_peir_sc_pot',data={x:tt,y:thd_455_dat.sc_pot}
		  endif
		endif
		common thd_456,thd_456_ind,thd_456_dat 
		if n_elements(thd_456_dat) ne 0 then begin
		  if thd_456_ind ne -1 then begin
			tt=(thd_456_dat.time+thd_456_dat.end_time)/2. & time=tt+tshft_mom*thd_456_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thd_456_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thd_peib_sc_pot',data={x:tt,y:thd_456_dat.sc_pot}
		  endif
		endif
		common thd_457,thd_457_ind,thd_457_dat 
		if n_elements(thd_457_dat) ne 0 then begin
		  if thd_457_ind ne -1 then begin
			tt=(thd_457_dat.time+thd_457_dat.end_time)/2. & time=tt+tshft_mom*thd_457_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thd_457_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thd_peef_sc_pot',data={x:tt,y:thd_457_dat.sc_pot}
		  endif
		endif
		common thd_458,thd_458_ind,thd_458_dat 
		if n_elements(thd_458_dat) ne 0 then begin
		  if thd_458_ind ne -1 then begin
			tt=(thd_458_dat.time+thd_458_dat.end_time)/2. & time=tt+tshft_mom*thd_458_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thd_458_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thd_peer_sc_pot',data={x:tt,y:thd_458_dat.sc_pot}
		  endif
		endif
		common thd_459,thd_459_ind,thd_459_dat 
		if n_elements(thd_459_dat) ne 0 then begin
		  if thd_459_ind ne -1 then begin
			tt=(thd_459_dat.time+thd_459_dat.end_time)/2. & time=tt+tshft_mom*thd_459_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			thd_459_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'thd_peeb_sc_pot',data={x:tt,y:thd_459_dat.sc_pot}
		  endif
		endif
	endif else if probes[i] eq 'e' then begin
		common the_454,the_454_ind,the_454_dat 
		if n_elements(the_454_dat) ne 0 then begin
		  if the_454_ind ne -1 then begin
			tt=(the_454_dat.time+the_454_dat.end_time)/2. & time=tt+tshft_mom*the_454_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			the_454_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'the_peif_sc_pot',data={x:tt,y:the_454_dat.sc_pot}
		  endif
		endif
		common the_455,the_455_ind,the_455_dat 
		if n_elements(the_455_dat) ne 0 then begin
		  if the_455_ind ne -1 then begin
			tt=(the_455_dat.time+the_455_dat.end_time)/2. & time=tt+tshft_mom*the_455_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			the_455_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'the_peir_sc_pot',data={x:tt,y:the_455_dat.sc_pot}
		  endif
		endif
		common the_456,the_456_ind,the_456_dat 
		if n_elements(the_456_dat) ne 0 then begin
		  if the_456_ind ne -1 then begin
			tt=(the_456_dat.time+the_456_dat.end_time)/2. & time=tt+tshft_mom*the_456_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			the_456_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'the_peib_sc_pot',data={x:tt,y:the_456_dat.sc_pot}
		  endif
		endif
		common the_457,the_457_ind,the_457_dat 
		if n_elements(the_457_dat) ne 0 then begin
		  if the_457_ind ne -1 then begin
			tt=(the_457_dat.time+the_457_dat.end_time)/2. & time=tt+tshft_mom*the_457_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			the_457_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'the_peef_sc_pot',data={x:tt,y:the_457_dat.sc_pot}
		  endif
		endif
		common the_458,the_458_ind,the_458_dat 
		if n_elements(the_458_dat) ne 0 then begin
		  if the_458_ind ne -1 then begin
			tt=(the_458_dat.time+the_458_dat.end_time)/2. & time=tt+tshft_mom*the_458_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			the_458_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'the_peer_sc_pot',data={x:tt,y:the_458_dat.sc_pot}
		  endif
		endif
		common the_459,the_459_ind,the_459_dat 
		if n_elements(the_459_dat) ne 0 then begin
		  if the_459_ind ne -1 then begin
			tt=(the_459_dat.time+the_459_dat.end_time)/2. & time=tt+tshft_mom*the_459_dat.delta_t
			pot = interp(tmp.y,tmp.x,time)
			the_459_dat.sc_pot=scale*(pot+offset) > min_pot
			store_data,'the_peeb_sc_pot',data={x:tt,y:the_459_dat.sc_pot}
		  endif
		endif
	endif

endfor
end
